unit CONTOURS; {Unit 10 of CONTEXT.PAS }

interface

  uses
    GLOBALS, SCREEN, POST, Graph, Dos, Crt;

  function xSection (var hgt1, hgt2, xpt1, xpt2: real): real;

  function ySection (var hgt1, hgt2, ypt1, ypt2: real): real;

  procedure GetIntersectPoint (i, j: integer;
                  intersectSide: integer;
                  var x1, y1: real;
                  var xVector, yVector: vector;
                  var hgt: fiveRvec);

  procedure DrawSegment (var xR1, yR1, xR2, yR2: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure DrawPostSeg (var F: text;
                  var xR1, yR1, xR2, yR2: real;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure PlotTheMinima (var F: text;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX);

  procedure ContourPlot (var F: text;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var D: contourMatrix);

implementation

{_________________________________________________}

  function xSection (var hgt1, hgt2, xpt1, xpt2: real): real;

{This function interpolates a point into a horizontal side of the cell}
{which is side (1) or (3)}

  begin

    xSection := (xpt1 * hgt2 - xpt2 * hgt1) / (hgt2 - hgt1);
  end; {xSection}

{_________________________________________________}

  function ySection (var hgt1, hgt2, ypt1, ypt2: real): real;

{This function interpolates a point into a vertical side of the cell}
{whichis side (2) or (4)}

  begin
    ySection := (ypt1 * hgt2 - ypt2 * hgt1) / (hgt2 - hgt1);
  end; {ySection}

{_________________________________________________}

  procedure GetIntersectPoint (i, j: integer;
                  intersectSide: integer;
                  var x1, y1: real;
                  var xVector, yVector: vector;
                  var hgt: fiveRvec);

  begin

    case intersectSide of
      1, 5: 
        begin
          x1 := xSection(hgt[1], hgt[2], xVector[j], xVector[j + 1]);
          y1 := yVector[i];
        end;
      2: 
        begin
          x1 := xVector[j + 1];
          y1 := ySection(hgt[2], hgt[3], yVector[i], yVector[i + 1]);
        end;
      3: 
        begin
          x1 := xSection(hgt[3], hgt[4], xVector[j + 1], xVector[j]);
          y1 := yVector[i + 1];
        end;
      4: 
        begin
          x1 := xVector[j];
          y1 := ySection(hgt[4], hgt[1], yVector[i + 1], yVector[i]);
        end;
    end; {cases}

  end; {GetIntersectPoint}

{_________________________________________________}

  procedure DrawSegment (var xR1, yR1, xR2, yR2: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      HR1, HR2, VT1, VT2: integer;
      temp: real;

  begin
    with SCREENBOX, dataBOX do
      begin
        temp := BOXHGT * (yR1 - yMin) / yRange;
        VT1 := Y2 - Round(temp);
        temp := BOXWDT * (xR1 - xMin) / xRange;
        HR1 := X1 + Round(temp);

        temp := BOXHGT * (yR2 - yMin) / yRange;
        VT2 := Y2 - Round(temp);
        temp := BOXWDT * (xR2 - xMin) / xRange;
        HR2 := X1 + Round(temp);

      end;
    MoveTo(HR1, VT1);
    LineTo(HR2, VT2);

  end; {DrawSegment}

{_________________________________________________}

  procedure DrawPostSeg (var F: text;
                  var xR1, yR1, xR2, yR2: real;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX);

    var
      VT1, VT2, HR1, HR2, temp: real;

  begin

{BOXWDT should be an IBOX parameter}

    with POSTBOX, dataBOX do
      begin
        temp := BOXHGT * (yR1 - yMin) / yRange;
        VT1 := Y1 + temp;
        temp := BOXWDT * (xR1 - xMin) / xRange;
        HR1 := X1 + temp;

        temp := BOXHGT * (yR2 - yMin) / yRange;
        VT2 := Y1 + temp;
        temp := BOXWDT * (xR2 - xMin) / xRange;
        HR2 := X1 + temp;

      end;

    Writeln(F, HR1 : 3 : 2, '  ', VT1 : 3 : 2, '  moveto');
    Writeln(F, HR2 : 3 : 2, '  ', VT2 : 3 : 2, '  lineto');

  end; {DrawPostSeg}

{_________________________________________________}

  procedure PlotTheMinima (var F: text;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX);
    var
      temp, pH, pV: real;
      HR, VT, radius, k: integer;

  begin { PlotTheMinima}
    radius := 2;

    for k := 1 to dataBOX.nOfMins do
      begin {k}

        with SCREENBOX, dataBOX do
          begin
            temp := BOXHGT * (minPoint[k].y - yMin) / yRange;
            VT := Y2 - Round(temp);
            temp := BOXWDT * (minPoint[k].x - xMin) / xRange;
            HR := X1 + Round(temp);
            Circle(HR, VT, radius);
          end; {with SCREENBOX, dataBOX}
      end; {k}

    if PSPict and (postFileName <> '') then
      with POSTBOX, dataBOX do
        begin
          Writeln(F, 'stroke');
          Writeln(F, 'newpath');

          for k := 1 to dataBOX.nOfMins do
            begin {k}

              temp := BOXHGT * (minPoint[k].y - yMin) / yRange;
              pV := Y1 + temp;
              temp := BOXWDT * (minPoint[k].x - xMin) / xRange;
              pH := X1 + temp;
              Writeln(F, pH : 6 : 4, '  ', pV : 6 : 4, '  ', radius : 1, '  blackdot');

            end; {k}

        end; {with POSTBOX, dataBOX}

  end; { PlotTheMinima}

{_________________________________________________}

  procedure ContourPlot (var F: text;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var D: contourMatrix);

    var
      hgt: fiveRVec;
      sign, intersect: fiveIVec;
      maxHgt, minHgt, x1, y1, x2, y2: real;
      i, j, k, p, q: integer;

  begin

    with dataBOX do
      begin

        for i := 0 to iMax - 1 do
          begin {i}
            for j := 0 to jMax - 1 do
              begin {j}

                maxHgt := D[i, j];
                minHgt := D[i, j];
                for p := 0 to 1 do
                  for q := 0 to 1 do
                    begin {p, q}
                      if D[i + p, j + q] > maxHgt then
                        maxHgt := D[i + p, j + q];
                      if D[i + p, j + q] < minHgt then
                        minHgt := D[i + p, j + q]
                    end;{p, q}

{This determines the maximum and minumum of the data values }
{within the square with vertices (i, j), (i, j+1),(i+1, j+1),(i, j+1).}

                for k := 1 to levels do
                  begin {k}

                    for q := 1 to 5 do
                      hgt[q] := 0.0;

                    if (maxHgt > zVector[k]) and (minHgt < zVector[k]) then
                      begin {if}
                        hgt[1] := D[i, j] - zVector[k];
                        hgt[2] := D[i, j + 1] - zVector[k];
                        hgt[3] := D[i + 1, j + 1] - zVector[k];
                        hgt[4] := D[i + 1, j] - zVector[k];
                        hgt[5] := hgt[1];
                      end; {if}

{The sides of the square are numbered by p=1, 5: North, 2: East, 3: South, 4: West.}
{The vertices are numbered  [i, j] =1, 5, [i, j+1] = 2,  [i +1, j+1] = 3,  [i +1, j] = 4.}

                    for q := 1 to 5 do
                      begin {q}
                        if hgt[q] < 0.0 then
                          sign[q] := -1;
                        if hgt[q] >= 0.0 then
                          sign[q] := 1;
                      end; {q}

{sign[q] := -1, if the vertex is below the contour, sign[q] := +1, if it is above}

                    p := 0;
                    for q := 1 to 4 do
                      begin {q}
                        if sign[q] * sign[q + 1] < 0 then
                          begin {if}
                            p := p + 1;
                            intersect[p] := q;
                          end;{if}
                      end; {q}

{Intersect[p] =q indicates that the pth intersection with the}
{contour occurs on the side of the square that is indexed by q.}

{either p = 2, if there is only one line segement cutting the box, or }
{p =4. if there are four, which would have a four common points on}
{the perimeter of the cell}

                    if p > 0 then
                      begin {if}
                        GetIntersectPoint(i, j, intersect[1], x1, y1, xVector, yVector, hgt);
                        GetIntersectPoint(i, j, intersect[2], x2, y2, xVector, yVector, hgt);
                        DrawSegment(x1, y1, x2, y2, dataBOX, SCREENBOX);
                        if PSPict and (postFileName <> '') then
                          DrawPostSeg(F, x1, y1, x2, y2, dataBOX, POSTBOX)
                      end; {if}
                  end; {k}
              end; {j}
          end; {i}

      end; {with dataBOX}

    if plotMinima then
      PlotTheMinima(F, SCREENBOX, POSTBOX, dataBOX);

{CLOSE(F) is in ORGANISE;}
  end; {ContourPlot}

{_________________________________________________}

end. {CONTOURS: Unit 10 of CONTEXT.PAS }

