unit GLOBALS; {Unit 1 of CONTEXT.PAS}

interface

  const
    TAB = 9;
    SPACE = 32;
    maxArray = 500;
    discrete = 1;
    continuous = 2;
    partial = 1;
    ordinary = 2;

  type
    complex = record
        re, im: real;
      end;

  type
    polar = record
        argu, modu: real;
      end;

  type
    points = record
        x, y: real
      end;

  type
    vector = array[0..64] of real;

  type
    shortVector = array[0..20] of real;

  type
    longVector = array[0..500] of real;

  type
    ivector = array[0..20] of integer;

  type
    fiveIvec = array[0..5] of integer;

  type
    fiveRvec = array[0..6] of real;

  type
    tenRVector = array[0..15] of real;

  type
    complexVector = array[0..20] of complex;

  type
    polarVector = array[0..20] of polar;

  type
    matrix = array[0..20, 0..20] of real;

  type
    sPoint = array[0..10] of real; {sPoint}

  type
    sMatrix = array[0..10] of sPoint;

  type
    stringVector = array[1..4] of string;

  type
    contourMatrix = array[0..64, 0..64] of real;

  type
    pointsVector = array[0..4] of points;

  type
    triplet = record
        funcValu: real;
        alphaValu: real;
        betaValu: real;
        leastRow: integer;
        leastcol: integer;
      end;

  type
    tripletVector = array[0..16] of triplet;

  type
    tripletMatrix = array[0..4, 0..4] of triplet;

  type
    IBOX = record
        xScale: boolean; {Horizontal scale below the bounding box}
        yScale: boolean; {Vertical scale to the left of the bounding box}
        xAxis: boolean; {Horizontal axis through y = 0}
        yAxis: boolean; {Vertical axis through x = 0}
        doFrame: boolean;
        doMarks: boolean;
        hBorders: boolean; {horizontal borders above and below the plot}
        X0: integer; {The horizontal value of the Graphics Origin}
        Y0: integer; {The vertical Value of the Graphics Origin}
        X1: integer; {Left of the Graphics Bounding Box}
        Y1: integer; {Top or Bottom  of the Graphics Bounding Box}
        X2: integer; {Right of the  GraphicsBounding Box}
        Y2: integer; {Bottom or Top of Graphics the Bounding Box}
        XC: integer; {Horizontal value of the centre of window}
        YC: integer; {Vertical value of the centre of window}
        BOXWDT: integer; {Width of the Bounding Box}
        BOXHGT: integer; {Height of the Bounding Box}
        minPoint: pointsVector; {prespecified mimimising points}
        nOfMins: integer; {number of mimimising points}
      end;

  type
    rBOX = record
        NOPUNCT: integer; {number of data points}
        xMax: real; {the maximum value of x in the dataVector}
        xMin: real; {the minmum value  of x in the dataVector}
        yMax: real; {the maximum value of y in the dataVector}
        yMin: real; {the minimum value of y in the dataVector}
        yMean: real; {the mean value of y in the dataVector}
        zMax: real; {matrixMax}
        zMin: real; {matrixMin}
        zMini: fiveIvec;
        zMinj: fiveIvec;
        xVector: vector; {horizontal divisions of the data grid}
        yVector: vector; {vertical divisions of the data grid}
        zVector: tenRVector; {contour levels}
        levels: integer; {number of contour levels}
        xRange: real; {xMax - xMin}
        yRange: real; {yMax - yMin}
        zRange: real; {zMax - zMin}
        yUB: real; {yUB >= yMax: an upper bound for y}
        yLB: real; {yLB <= yMin: a lower bound for y}
        xLB: real; {xLB <= xMin: a left (lower) bound for x}
        xRB: real; {xRB >= xMax: a right (upper) bound for x}
        iMax, jMax: integer; {number of data grid divisions}
        deltax: real; {increment of a between adjacent cells}
        deltay: real; {increment of b between adjacent cells}
        numbOfRows: integer; {number of row divisions of the plotMatrix}
        numbOfCols: integer; {number of column divisions of the plotMatrix}
        minPoint: pointsVector; {prespecified minimizing point}
        nOfMins: integer; {number of prespecified minimizing point}
      end;

  type
    ARMARecord = record
        noModel: boolean;
        ordersSpecified: boolean;
        parametersFound: boolean;
        rootsFound: boolean;
        aCovarsFound: boolean;
        varEpsilon: real; {variance of white-noise forcing function}
        p: integer; {degree of the denominator}
        pr: integer; {no of pairs of complex denominator roots}
        q: integer; {degree of the numerator}
        qr: integer; {no pairs of complex numerator roots}
        alpha: shortVector; {coefficents of the denominator}
        beta: shortVector; {coefficents of the numerator}
        poles: complexVector; {Cartesian denominator roots}
        zeros: complexVector; {Cartesian numerator roots}
        polarPoles: polarVector; {polar denominator roots}
        polarZeros: polarVector; {polar numerator roots}
        aCovar: shortVector; {autocovariances}
      end;

{in the case of an discrete-time ARMA model the poles and zeros are the}
{inverse roots of the lag polynomial, which must lie inside the unit circle}

{Graphics Prameters}
  var
    SCREENBOX, POSTBOX: IBOX;
    graphicsOn, noSCREENBOX: boolean;
    currentMode: integer;
    figNumber: integer;
    D: contourMatrix;
    postFileName: string;
    F: text;

  var
    discontent: integer; {1 => discrete, 2 => continuous}
    ARMASpec, CARMASpec, LSDESpec, EDLMSpec: ARMARecord;
    cellMinima: tripletMatrix;
    SortedMinima: tripletVector;
    allsDone, PSPict, TeXFormat, noModelParams, noPageParams: boolean;
    dataBOX: rBOX;
    noContourParams, noDataMatrix, noContourMatrix, plotMinima: boolean;

  procedure InitialiseARMARecord (var theRecord: ARMARecord);

  procedure InitialiseAnIBOX (var GraphBOX: IBOX);

implementation

{________________________________________________}

  procedure InitialiseARMARecord (var theRecord: ARMARecord);

  begin

    with theRecord do
      begin
        noModel := true;
        ordersSpecified := false;
        parametersFound := false;
        rootsFound := false;
        aCovarsFound := false;
        varEpsilon := 1.0; {variance of white-noise forcing function}
        p := 0; {degree of the denominator}
        pr := 0; {no of complex denominator roots}
        q := 0; {degree of the numerator}
        qr := 0; {no of complex numerator roots}
      end; {with theRecord}

  end; {InitialiseARMARecord}

{_________________________________________________}

  procedure InitialiseAnIBOX (var graphBOX: IBOX);

  begin
    with graphBOX do
      begin
        xScale := true;
        yScale := true;
        xAxis := false;
        yAxis := false;
        hBorders := false;
        doFrame := true;
        doMarks := true;
      end;
  end; {InitialiseAnIBOX}

{_________________________________________________}

end. {GLOBALS: Unit 1 of CONTEXT.PAS}



