program CONTEXT; {MAIN Unit of CONTEXT.PAS}

  uses
    GLOBALS, UTILS, ORGANISE, Dos, Crt, Graph;

  var
    theMenuItem: integer;

begin
  SetTheText;

  Writeln('    THE CONTOUR MAP FOR CONCRETE.PAS');

  Writeln;
  Writeln('    The purpose of this program is to generate a matrix of values that will');
  Writeln('    be supplied to a program for plotting the contour map of the criterion');
  Writeln('    function that we desire to optimise.');

  Writeln;
  Writeln('    The function in question is the sum of squares of differences between');
  Writeln('    the leading autocovariances of a discrete-time ARMA(2, 1) model with ');
  Writeln('    complex poles and its continuous time LSDE counterpart.');

  Writeln;
  Writeln('    The poles of the ARMA model are translated directly to the poles of the ');
  Writeln('    LSDE. The two real-valued parameters (a, b) determine the moving average');
  Writeln('    coefficients and the variance of the forcing function of the LSDE. These');
  Writeln('    are also the arguments of the criterion function.');

  Writeln;
  Writeln('    (We should note that there are only two free parameters to be determined.');
  Writeln('    Either  the leading  coefficient  of the  moving average  polynomial is');
  Writeln('    normalised at unity, or else the variance is normalised at unity.');

  Writeln;
  Writeln('    You will be asked to specify the ARMA model. Then you will be asked to');
  Writeln('    place bounds on the values of (a, b) and to specify the number of elements');
  Writeln('    in the matrix of values of the criterion function.');

  Writeln;
  Writeln(' ' : 68, '<RETURN>');
  Readln;

  graphicsOn := false;
  noSCREENBOX := true;
  SCREENBOX.XC := 0;
  SCREENBOX.YC := 0;
  noPageParams := true;
  noContourParams := true;
  noContourMatrix := true;
  ARMASpec.noModel := true;
  plotMinima := false;
  allsDone := false;
  InitialiseAnIBOX(SCREENBOX);
  InitialiseAnIBOX(POSTBOX);

  repeat
    ClrScr;

    Writeln;
    Writeln('    THE CONTOUR MAP FOR CONCRETE.PAS');
    Writeln;
    Writeln('       1.  Get Page Parameters');
    Writeln;
    Writeln('       2.  Specify the Parameters of the ARMA Model');
    Writeln;
    Writeln('       3.  Test a Specific Point in the Function Space');
    Writeln;
    Writeln('       4.  Specify the Parameters of the Contour Plot');
    Writeln;
    Writeln('       5.  Generate a Matrix of the Criterion Function Values');
    Writeln;
    Writeln('       6.  Locate Local Minima amongst the Function Values');
    Writeln;
    Writeln('       7.  Plot the Contour Lines');
    Writeln;
    Writeln('       8.  SAVE and EXIT');

    Writeln;
    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, 8, theMenuItem);

    case theMenuItem of
      1: 
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TexFormat);
      2: 
        GetModelParams(SCREENBOX, POSTBOX, ARMASpec, LSDESpec); {in ORGANISE}
      3: 
        TestAPoint(ARMASpec, LSDESpec); {in ORGANISE}
      4: 
        GetContourParameters(SCREENBOX, POSTBOX, dataBOX, D); {in ORGANISE}
      5: 
        CreatePlotMatrix(ARMASpec, LSDESpec, dataBOX, D); {in ORGANISE}
      6: 
        MatrixSearch(ARMASpec, dataBOX, cellMinima, SortedMinima); {in ORGANISE}
      7: 
        PlotContourLines(F, SCREENBOX, POSTBOX, ARMASpec, LSDESpec, dataBOX, D); {In ORGANISE}
      8: 
        SaveAndExit(dataBOX, allsDone); {in ORGANISE}
    end; {case}

  until allsDone;

  RevertText;

end.

{_________________________________________________}

{MAIN Unit of CONTEXT.PAS}

