unit ORGANISE; {Unit 11 of CONTEXT.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, ACQUIRE, CACOVARS, DACOVARS, SCREEN, POST, CONTOURS, Graph, Dos, Crt;

  function Funct (var alpha, beta: real;
                  var ARMASpec, LSDESpec: ARMARecord;
                  var lags: integer): real;

  procedure GetModelParams (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord);

  procedure TestAPoint (var ARMASpec, LSDESpec: ARMARecord);

  procedure GetTheMinima (var dataBOX: rBOX);

  procedure GetTheDataBounds (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX);

  procedure InvertTheData (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var D: contourMatrix);

  procedure GetContourParameters (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var D: contourMatrix);

  procedure AltGetContourParameters (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX);

  procedure CreatePlotMatrix (var ARMASpec, LSDESpec: ARMARecord;
                  var dataBOX: rBOX;
                  var D: contourMatrix);

  procedure SortThem (var dataBOX: rBOX;
                  var SortedMinima: tripletVector;
                  cellMinima: tripletMatrix);

  procedure SearchInCells (var dataBOX: rBOX;
                  var cellMinima: tripletMatrix);

  procedure MatrixSearch (var ARMASpec: ARMARecord;
                  var dataBOX: rBOX;
                  var cellMinima: tripletMatrix;
                  var SortedMinima: tripletVector);

  procedure PlotContourLines (var F: text;
                  var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord;
                  var dataBOX: rBOX;
                  var D: contourMatrix);

  procedure SaveAndExit (var dataBOX: rBOX;
                  var allsDone: boolean);

implementation

{_________________________________________________}

  function Funct (var alpha, beta: real;
                  var ARMASpec, LSDESpec: ARMARecord;
                  var lags: integer): real;

{We shall modify this by passing ARMASpec as a parameter of Funct}
{This will replace gammaD as an internal parameter}

    var
      kappa: complexVector;
      gammaC, gammaD: shortVector;
      measure: real;
      i: integer;


  begin

    for i := 0 to lags do
      gammaD[i] := ARMASpec.ACovar[i];

    for i := 1 to lags do
      kappa[i] := LSDESpec.poles[i];

    RealcCovar(alpha, beta, kappa, gammaC, lags);

    measure := 0.0;
    for i := 0 to lags do
      measure := measure + Sqr(gammaC[i] - gammaD[i]);
    Funct := measure;

  end; {Funct}

{_________________________________________________}

  procedure GetModelParams (var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord);

    var
      stability, acquireNewModel: boolean;
      DorC: char;

  begin
    ClrScr;

    Writeln;
    Writeln('    SPECIFY THE MODEL PARAMETERS');
    Writeln;

    Writeln('    Specify the parameters of a stationary discrete-time ARMA model');
    Writeln;
    Writeln(' ' : 59, '<RETURN>');
    Readln;

    DorC := 'D';
    stability := True;
    acquireNewModel := True;

    if (DorC = 'D') then
      begin {discrete}
        discontent := discrete;
        if ARMASpec.noModel = false then
          begin
            SysBeep(3);
            Writeln('    You have already specified a discrete-time model.');
            YesOrNo('    Do you wish to discard it', acquireNewModel);
          end;
        if acquireNewModel then
          begin
            InitialiseARMARecord(ARMASpec);
            InitialiseARMARecord(LSDESpec);
            InitialiseARMARecord(CARMASpec);
            InitialiseARMARecord(EDLMSpec);
            ARMAacquisition(ARMASpec, stability); {in ACQUIRE}
          end;
      end; {discrete}

    noModelParams := false;

  end; {GetModelParams}

{_________________________________________________}

  procedure TestAPoint (var ARMASpec, LSDESpec: ARMARecord);

    var
      degToRad, a, b, testValue, temp: real;
      lags, i: integer;
      interchange: boolean;
      gamma: shortVector;
      complexAlpha: complexVector;

  begin
    degToRad := ArcTan(1.0) / 45.0;

    if ARMASpec.noModel then
      GetModelParams(SCREENBOX, POSTBOX, ARMASpec, LSDESpec);

    lags := ARMASpec.p + ARMASpec.q;
    if (ARMASpec.aCovarsFound = false) then
      ARMACovariances(ARMASpec, gamma, lags);

    if (LSDESpec.rootsFound = false) then
      begin {if}
        for i := 1 to ARMASpec.p do
          begin {i}
            LSDESpec.polarPoles[i].argu := ARMASpec.polarPoles[i].argu * degToRad;
            LSDESpec.polarPoles[i].modu := ARMASpec.polarPoles[i].modu;
            ComplexLog(LSDESpec.polarPoles[i], LSDESpec.poles[i]);
          end; {i}
        LSDESpec.p := ARMASpec.p;
        LSDESpec.rootsFound := true;
        InverseRootsToCoeffs(LSDESpec.p, LSDESpec.poles, complexAlpha);
      end; {if}

    for i := 0 to ARMASpec.p do
      LSDESpec.alpha[i] := complexAlpha[i].re;

    ClrScr;

    Writeln;
    Writeln('    TEST A SPECIFIC POINT');
    Writeln;
    Writeln('    Specify a value for the parameter alpha');
    ReadANumber(a);

    Writeln('    Specify a value for the parameter beta');
    ReadANumber(b);
    testValue := Funct(a, b, ARMASpec, LSDESpec, lags);
    Writeln('    Function value = ', testValue : 6 : 4);
    Writeln;

    Writeln('    The MOVING-AVERAGE PARAMETERS of THE LSDE');
    Writeln;
    with LSDESpec do
      begin
        beta[0] := 2 * a;
        beta[1] := 2 * (a * poles[1].re + b * poles[1].im);
        Writeln('    theta[0] = ', beta[0] : 6 : 4);
        Writeln('    theta[1] = ', beta[1] : 6 : 4);
        Readln;
      end;

    Writeln;
    YesOrNo('    Do you wish to interchange the values of a and b', interchange);
    if interchange then
      begin {if}
        temp := a;
        a := b;
        b := temp;
        testValue := Funct(a, b, ARMASpec, LSDESpec, lags);
        Writeln('    Function value = ', testValue : 6 : 4);
        Readln;
      end; {if}


  end; {TestAPoint}

{_________________________________________________}

  procedure GetTheMinima (var dataBOX: rBOX);

    var
      k: integer;
      moreMins: boolean;

  begin
    ClrScr;

    Writeln;
    Writeln('    PLOT MINIMUM VALUES ON THE CONTOUR MAP');
    Writeln;
    Writeln('    Since the criterion  function is virtually flat in the vicinity of the');
    Writeln('    mimimising values, it is difficult to locate these values by searching');
    Writeln('    the data grid.  The minimising  values are located  accurately  by the');
    Writeln('    adjunct  program  CONCRETE.PAS, which  uses the Nelder-Mead procedure.');
    Writeln('    These values can be imported into the present program.');

    Writeln;
    YesOrNo('    Do you wish to plot the known minimising values', plotMinima);

    if plotMinima then
      begin {plotMinima: global}

        moreMins := true;
        k := 0;
        repeat
          with dataBOX do
            begin
              k := k + 1;
              Writeln('    Specify the  x-value of minimum');
              ReadANumber(minPoint[k].x);
              Writeln('    Specify the  y-value of minimum');
              ReadANumber(minPoint[k].y);
              YesOrNo('    Do you wish to plot more minimising values', moreMins);
              nOfMins := k;
            end; {with dataBOX}
        until (moreMins = false);

      end; {if plotMinima}

  end; {GetTheMinima}

{_________________________________________________}

  procedure GetTheDataBounds (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX);

  begin

    with dataBOX do
      begin

        Writeln('    Specify the lower bound on the parameter a');
        ReadANumber(xMin);

        Writeln('    Specify the upper bound on the parameter a');
        ReadANumber(xMax);

        Writeln('    Specify the lower bound on the parameter b');
        ReadANumber(yMin);

        Writeln('    Specify the upper bound on the parameter b');
        ReadANumber(yMax);

        Writeln('    Specify the limit iMax of the row index of the data matrix');
        GetBoundedInteger(0, 64, iMax);

        Writeln('    Specify the limit jMax of the column index of the data matrix');
        GetBoundedInteger(0, 64, jMax);

        Writeln('    How many contour levels do you require?');
        GetBoundedInteger(2, 15, levels);

        xRange := xMax - xMin;
        yRange := yMax - yMin;

        xLB := xMin;
        xRB := xMax;

        deltax := xRange / jMax;
        deltay := yRange / iMax;

        if plotMinima = false then
          GetTheMinima(dataBOX);

        noContourParams := false;
        noContourMatrix := true;
      end; {with dataBOX}

  end; {GetTheDataBounds}

{_________________________________________________}

  procedure InvertTheData (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var D: contourMatrix);
    var
      R, Rbound, alpha, zinc: real;
      i, j, k: integer;

  begin
    ClrScr;

    Writeln;
    Writeln('    INVERT THE THE DATA VALUES');
    Writeln;
    Writeln('    To highlight the minimum points of criterion function, we may replace');
    Writeln('    the  surface values z by  q = 1/(z + a), where  a is a small positive');
    Writeln('    number that prevents a  division by zero. We  set a = (X - RM)/(R-1),');
    Writeln('    where M = min (z), X = max(z)  and R = max(q)/\min(q). You  are asked');
    Writeln('    to specify a value for R.');
    Writeln;

    with dataBOX do
      begin
        Rbound := (zMax - zMin) / zMin;
        Writeln('    What is the value of R?');
        ReadBoundedNumber(1, RBound, R);
        alpha := (zMax - R * zMin) / (R - 1);

        zMax := 0.0;
        zMin := 999.9;
        for i := 0 to iMax do
          for j := 0 to jMax do
            begin
              D[i, j] := 1.0 / (alpha + D[i, j]);
              if D[i, j] > zMax then
                zMax := D[i, j];
              if D[i, j] < zMin then
                zMin := D[i, j];
            end; {ij}

        zInc := (zMax - zMin) / (levels + 1);

        WRITELN;
        WRITELN('zMin  =  ', zMin : 6 : 4);
        WRITELN('Max =  ', zMax : 6 : 4);
        WRITELN;
        for k := 1 to levels do
          begin {k}
            zVector[k] := zMin + k * zInc;
            WRITELN('    zVector[', k : 2, ']  =  ', zVector[k] : 6 : 4);
          end; {k}

      end; {with dataBOX}
    WRITELN;
    WRITELN(' ' : 30, '<RETURN>');
    READLN;

  end; {InvertTheData }

{_________________________________________________}

  procedure GetContourParameters (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var D: contourMatrix);
    var
      theMenuItem: Integer;

  begin

    if noContourMatrix then
      begin
        ClrScr;

        Writeln;
        Writeln('    PARAMETERS OF THE CONTOUR PLOT');
        Writeln;
        Writeln('    A contour  map will be generated over a  rectangle that is determined by');
        Writeln('    choosing intervals  for the arguments a and b of the criterion function.');
        Writeln('    The minimisation of this function will match the autocovariance function');
        Writeln('    of the LSDE to that of the specified ARMA model.');
        Writeln;
        Writeln('    The argument  a corresponds to  the horizontal x-axis in the  plane. The');
        Writeln('    argument b  corresponds to  the vertical axis in the plane. The z-values');
        Writeln('    of the function correspond to heights above  the plane. A grid is placed');
        Writeln('    over the planar rectangle and the z-values are calculated  at the points');
        Writeln('    of  intersection. The horizontal index  of the grid is j = 0 to jMax and');
        Writeln('    the vertical index is i = 0 to iMax.');

        Writeln;
        Writeln(' ' : 68, '<RETURN>');
        Readln;

        GetTheDataBounds(SCREENBOX, POSTBOX, dataBOX);
      end
    else
      begin {else}
        repeat
          ClrScr;

          Writeln;
          Writeln('    PARAMETERS OF THE CONTOUR PLOT');
          Writeln;
          Writeln('      1.  Respecify the Contour Parameters');
          Writeln;
          Writeln('      2.  Invert the z-values');
          Writeln;
          Writeln('      3.  EXIT this Procedure');
          Writeln;
          Writeln;
          Writeln('                      Select a number and press RETURN');

          GetBoundedInteger(1, 3, theMenuItem);

          case theMenuItem of
            1: 
              GetTheDataBounds(SCREENBOX, POSTBOX, dataBOX);
            2: 
              InvertTheData(SCREENBOX, POSTBOX, dataBOX, D);
            3: 
              ; {EXIT this Procedure}
          end; {cases}
        until theMenuItem in [1, 3];
      end;{else}

  end; {GetContourParameters}

{_________________________________________________}

  procedure AltGetContourParameters (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX);

  begin
    ClrScr;

    Writeln;
    Writeln('    PARAMATERS OF THE CONTOUR PLOT');
    Writeln;

    with dataBOX do
      begin

        Writeln('    Specify the lower bound on the parameter a');
        ReadANumber(xMin);

        Writeln('    Specify the upper bound on the parameter a');
        ReadANumber(xMax);

        Writeln('    Specify the lower bound on the parameter b');
        ReadANumber(yMin);

        Writeln('    Specify the upper bound on the parameter b');
        ReadANumber(yMax);

        Writeln('    Specify the limit iMax of the row index of the data matrix');
        GetBoundedInteger(0, 64, iMax);

        Writeln('    Specify the limit jMax of the column index of the data matrix');
        GetBoundedInteger(0, 64, jMax);

        Writeln('    How many contour levels do you require?');
        GetBoundedInteger(2, 15, levels);

        xRange := xMax - xMin;
        yRange := yMax - yMin;

        xLB := xMin;
        xRB := xMax;

        deltax := xRange / jMax;
        deltay := yRange / iMax;

        noContourParams := false;
        noContourMatrix := true;
      end; {with dataBOX}


  end; {GetContourParameters}

 {_________________________________________________}

  procedure CreatePlotMatrix (var ARMASpec, LSDESpec: ARMARecord;
                  var dataBOX: rBOX;
                  var D: contourMatrix);

    var
      i, j, k, lags: integer;
      complexAlpha: complexVector;
      gamma: shortVector;
      zinc, alpha, beta, degToRad: real;

  begin
    degToRad := ArcTan(1.0) / 45.0;

    if ARMASpec.noModel then
      GetModelParams(SCREENBOX, POSTBOX, ARMASpec, LSDESpec);

    if noContourParams = true then
      GetContourParameters(SCREENBOX, POSTBOX, dataBOX, D);


{Create The discrete Covariance and put it into ARMASpec}

    lags := ARMASpec.p + ARMASpec.q;
    if (ARMASpec.aCovarsFound = false) then
      ARMACovariances(ARMASpec, gamma, lags);

{Translate the ARMA poles to the LSDE poles and compund them}
{to for the LSDE denomimnator}

    if (LSDESpec.rootsFound = false) then {OR SHOULD IT BE POLES FOUND ?}
      begin {if}
        for i := 1 to ARMASpec.p do
          begin {i}
            LSDESpec.polarPoles[i].argu := ARMASpec.polarPoles[i].argu * degToRad;
            LSDESpec.polarPoles[i].modu := ARMASpec.polarPoles[i].modu;
            ComplexLog(LSDESpec.polarPoles[i], LSDESpec.poles[i]);
          end; {i}
        LSDESpec.p := ARMASpec.p;
        LSDESpec.rootsFound := true;
        InverseRootsToCoeffs(LSDESpec.p, LSDESpec.poles, complexAlpha);
      end; {if}

    for i := 0 to ARMASpec.p do
      LSDESpec.alpha[i] := complexAlpha[i].re;

{Form the D matrix}

    with dataBOX do
      begin {with}
        zMax := 0.0;
        zMin := 999.9;
        for i := 0 to iMax do {for i = 0 to iMax}
          for j := 0 to jMax do {for j = 0 to jMax}
            begin {j}
              alpha := xMin + (deltax * j);
              beta := yMin + (deltay * i);
              D[i, j] := Funct(alpha, beta, ARMASpec, LSDESpec, lags);
              if D[i, j] > zMax then
                zMax := D[i, j];
              if D[i, j] < zMin then
                zMin := D[i, j];
              xVector[j] := alpha;
              yVector[i] := beta;
            end; {i, j}

        zInc := (zMax - zMin) / (levels + 1);

        for k := 1 to levels do
          begin {k}
            zVector[k] := zMin + k * zInc
          end; {k}

      end; {with dataBOX}

    noContourMatrix := false;

  end; {CreatePlotMatrix}

{_________________________________________________}

  procedure SortThem (var dataBOX: rBOX;
                  var SortedMinima: tripletVector;
                  cellMinima: tripletMatrix);

{observe that cellMinima is a local copy }

    var
      i, r, c, n: integer;
      leastRow, leastCol: integer;
      leastValue: real;

  begin

    with dataBOX do
      begin {with}
        n := numbOfRows * numbOfCols;
        for i := 1 to n do
          begin {i}
            leastValue := 999;
            for r := 1 to numbOfRows do
              for c := 1 to numbOfCols do
                begin {r,c}
                  if leastValue > cellMinima[r, c].funcValu then
                    begin {if}
                      leastValue := cellMinima[r, c].funcValu;
                      leastRow := r;
                      leastCol := c;
                    end; {if}
                end; {r,c}
            SortedMinima[i] := cellMinima[leastRow, leastCol];
            cellMinima[leastRow, leastCol].funcValu := 999;
          end; {i}
      end; {with}
  end; {SortThem}

{_________________________________________________}

  procedure SearchInCells (var dataBOX: rBOX;
                  var cellMinima: tripletMatrix);

    var
      cellWidth, cellDepth, i, j, r, c: integer;
      colBreak, rowBreak: iVector;

{alpha varies with the column indices c and j}
{beta varies with the row indices r and i}

  begin

    with dataBOX do
      begin
        cellWidth := jMax div numbOfCols;
        cellDepth := iMax div numbOfRows;

        colBreak[0] := 0;
        for c := 1 to numbOfCols do
          colBreak[c] := cellWidth * c;

        rowBreak[0] := 0;
        for r := 1 to numbOfRows do
          rowBreak[r] := cellDepth * r;

        for r := 1 to numbOfRows do
          for c := 1 to numbOfCols do
            begin {r, c}
              cellMinima[r, c].funcValu := 999;
              for i := rowBreak[r - 1] + 1 to rowBreak[r] do
                for j := colBreak[c - 1] + 1 to colBreak[c] do
                  begin {i, j}
                    if D[i, j] < cellMinima[r, c].funcValu then
                      begin
                        cellMinima[r, c].funcValu := D[i, j];
                        cellMinima[r, c].alphaValu := xMin + j * deltax;
                        cellMinima[r, c].betaValu := yMin + i * deltay;
                      end;
                  end; {i, j}
            end;{r, c}

      end; {with}

  end;{SearchInCells}

{_________________________________________________}

  procedure MatrixSearch (var ARMASpec: ARMARecord;
                  var dataBOX: rBOX;
                  var cellMinima: tripletMatrix;
                  var SortedMinima: tripletVector);

    var
      theMenuItem, i, n: integer;
      skipIt: boolean;

  begin

    SkipIt := false;

    if ARMASpec.noModel then
      GetModelParams(SCREENBOX, POSTBOX, ARMASpec, LSDESpec);

    if noContourParams then
      GetContourParameters(SCREENBOX, POSTBOX, dataBOX, D);

    if noContourMatrix then
      CreatePlotMatrix(ARMASpec, LSDESpec, dataBOX, D);

    ClrScr;

    Writeln;
    Writeln('      SEARCH FOR LOCAL MINIMA');
    Writeln;
    Writeln('      This procedure  searches a matrix of function  values in pursuit of');
    Writeln('      starting values for a minimisation procedure. The matrix is divided');
    Writeln('      into of cells of equal sizes which are either 4, 9 or 16 in number,');
    Writeln('      and it displays both the  minimum  function values and their matrix');
    Writeln('      coordinates.');
    Writeln;
    Writeln('      1.  Search within quadrants');
    Writeln;
    Writeln('      2.  Search within 9 cells');
    Writeln;
    Writeln('      3.  Search within 16 cells');
    Writeln;
    Writeln('      4.  Skip the procedure and EXIT');
    Writeln;

    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, 4, theMenuItem);
    case theMenuitem of
      1: 
        begin
          dataBOX.numbOfRows := 2;
          dataBOX.numbOfCols := 2;
        end;
      2: 
        begin
          dataBOX.numbOfRows := 3;
          dataBOX.numbOfCols := 3;
        end;
      3: 
        begin
          dataBOX.numbOfRows := 4;
          dataBOX.numbOfCols := 4;
        end;
      4: 
        skipIt := true; {EXIT}
    end; {cases}

    if (skipIt = false) then
      begin {not skipIt}

        SearchInCells(dataBOX, cellMinima);
        SortThem(dataBOX, SortedMinima, cellMinima);

        with dataBOX do
          begin {with}
            n := numbofRows * numbofCols;

            for i := 1 to n do
              begin {i}
                Writeln(' rank = ', i : 2);
                Writeln(' Function value = ', SortedMinima[i].funcValu : 6 : 4);
                Writeln(' alpha value = ', SortedMinima[i].alphaValu : 6 : 4);
                Writeln(' beta value = ', SortedMinima[i].betaValu : 6 : 4);
                Writeln;
                if (i mod 4 = 0) or (i = n) then
                  begin {if}
                    Writeln(' ' : 40, '<RETURN>');
                    Readln;
                  end; {if}
              end; {i}
          end; {with}
      end; {not Skipit}

  end; {MatrixSearch}

{_________________________________________________}

  procedure PlotContourLines (var F: text;
                  var SCREENBOX, POSTBOX: IBOX;
                  var ARMASpec, LSDESpec: ARMARecord;
                  var dataBOX: rBOX;
                  var D: contourMatrix);

  begin

    if (noPageParams = true) then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TexFormat);

    if ARMASpec.noModel then
      GetModelParams(SCREENBOX, POSTBOX, ARMASpec, LSDESpec);

    if noContourParams then
      GetContourParameters(SCREENBOX, POSTBOX, dataBox, D);

    if noContourMatrix then
      CreatePlotMatrix(ARMASpec, LSDESpec, dataBOX, D); {}

    if PSPict then
      begin {PSPict}
        ClrScr;
        Writeln;
        Writeln('    CREATE THE POSTSCRIPT GRAPHIC');
        Writeln;
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {postFileName}
            Assign(F, postFileName);
            Rewrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            PostFrame(F, dataBOX, POSTBOX);
          end;
      end; {PSPict}

    ClrScr;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(dataBOX, SCREENBOX);
    ContourPlot(F, SCREENBOX, POSTBOX, dataBOX, D); {in CONTOURS.PAS}

    if PSPict and (postFileName <> '') then
      begin
        PostFooter(F, POSTBOX, TexFormat);
        CLose(F);
      end; {postFileName}

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

  end; {PlotContourLines}

{_________________________________________________}

  procedure SaveAndExit (var dataBOX: rBOX;
                  var allsDone: boolean);

    var
      i, j, k: integer;
      dataFileName: string;
      SaveIt: boolean;
      G: text;

  begin
    ClrScr;

    Writeln;
    Writeln('    EXIT THE PROGRAM OR RETURN');
    Writeln;

{Write the PlotMatrix to a file}
    if (noContourMatrix = false) then
      begin {noContourMatrix = false}
        YesOrNo('    Do you wish to save the data matrix', SaveIt);
        if SaveIt then
          begin {SaveIt}
            with dataBOX do
              begin {with dataBOX}
                Write('    Name the Data File = ');
                Readln(dataFileName);
                if dataFileName <> '' then
                  begin {if dataFileName <> ''}
                    Assign(G, dataFileName);
                    Rewrite(G);
                    for i := 0 to iMax do
                      begin {i}
                        Writeln(G);
                        for j := 0 to jMax do
                          Write(G, D[i, j] : 6 : 4, '  ');
                      end;{i}

                    Writeln(G);
                    for k := 1 to levels do
                      Writeln(G, '    zVector[', k : 2, '] = ', zVector[k] : 6 : 4, '  ');

                    Writeln(G);
                    for i := 0 to iMax do
                      Writeln(G, '    yVector[', i : 2, '] = ', yVector[i] : 6 : 4, '  ');

                    Writeln(G);
                    for j := 0 to jMax do
                      Writeln(G, '    xVector[', j : 2, '] = ', xVector[j] : 6 : 4, '  ');
                    Close(G);
                  end; {if: dataFileName}
              end; {with dataBOX}
          end; {SaveIt}
      end;{noContourMatrix = false}

    Sysbeep(3);

    YesOrNo('    Do you wish to EXIT the program', allsDone);

  end;{SaveAndExit}
{_________________________________________________}

end. {ORGANISE: Unit 11 of CONTEXT.PAS}

