unit POST; {Unit 8 of CONTEXT.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS;

  procedure PostHeader (var F: text;
                  POSTBOX: IBOX;
                  TeXFormat: boolean);

  procedure PostFooter (var F: text;
                  var POSTBOX: IBOX;
                  TexFormat: boolean);

  procedure PostMarks (var F: text;
                  XSTART, YSTART: real;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

  procedure PostFrame (var F: text;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure PostPlot (var F: text;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PostPlotMod (var F: text;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

implementation

{________________________________________________}

  procedure PostHeader (var F: text;
                  POSTBOX: IBOX;
                  TeXFormat: boolean);

    var
      BOXHGT, BOXWDT, INDENT: integer;
      LLx, LLy, URx, URy: integer;
      xOrigin, yOrigin: integer;

  begin
    xOrigin := 160;
    yOrigin := 560;

    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;

    if TeXFormat then
      Writeln(F, '\special{postscript')
    else
      begin
        LLx := xOrigin - 45;
        LLy := yOrigin - 25;
        URx := xOrigin + BOXWDT + 45;
        URy := yOrigin + BOXHGT + 10;
        Writeln(F, '%!PS-Adobe-2.0');
        Writeln(F, '%%Creator: SigmaPi');
        Writeln(F, '%%BoundingBox:', LLx : 3, '  ', LLy : 3, '  ', URx : 3, '  ', URy : 3);
      end;

    Writeln(F, '/xunits{1.00 mul} def');
    Writeln(F, '/yunits{1.00 mul} def');

    Writeln(F, '/showxhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x xunits y yunits moveto');
    Writeln(F, '0  5 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x xunits y yunits moveto');
    Writeln(F, '0 -20 rmoveto');
    Writeln(F, 'label dup stringwidth pop 2 div neg 0 rmoveto show} def');

    Writeln(F, '/showyhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x xunits y yunits moveto');
    Writeln(F, ' 5 0 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x xunits y yunits moveto');
    Writeln(F, '-15 -2 rmoveto');
    Writeln(F, 'label dup stringwidth pop neg 0 rmoveto show} def');

    Writeln(F, '/blackdot');
    Writeln(F, '{/radius exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, ' x  y  radius  0  360 arc fill');
    Writeln(F, '} def');

    Writeln(F, 'newpath');
    Writeln(F, '0.5 setlinewidth');

    if TeXFormat then
      begin
        INDENT := Round(((12 * 72) / 2.54 - BOXWDT) / 2);
        Writeln(F, INDENT : 2, '  ', -(BOXHGT + 15) : 2, '  translate');
      end
    else
      Writeln(F, xOrigin : 3, '  ', yOrigin : 3, ' translate ');

  end; {PostHeader}

{________________________________________________}

  procedure PostFooter (var F: text;
                  var POSTBOX: IBOX;
                  TexFormat: boolean);

    var
      BOXHGT: integer;

  begin
    Writeln(F, 'stroke');
    if TexFormat then
      begin
        Writeln(F, 'showpage}');
        BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
        Writeln(F, '\vbox to ', BOXHGT + 35 : 3, ' pt');
        Writeln(F, ' {\hrule width 12cm height 0cm depth 0cm \vfill}');
        Writeln(F, '\medskip');
        Write(F, ' This is the line for the caption');
        Writeln(F, '\par}');
        Writeln(F, '\medskip');
      end
    else
      begin
        Writeln(F, 'showpage');
        Writeln(F, '%%Trailer');
        Writeln(F, '%%EOF');
      end;

  end; {PostFooter}

{_________________________________________________}

  procedure PostMarks (var F: text;
                  XSTART, YSTART: real;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

    var
      toly, HR, VT: real;
      tolyString: string;

  begin {PostMarks}
    toly := firstMark;
    HR := XSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) + 0.5 * Abs(taly) do

      begin {while toly}
        tolyString := TalyToString(toly);
        if Axis = 'xScale' then
          begin
            Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showxhatch');
            HR := HR + STEP;
          end;

        if Axis = 'yScale' then
          begin
            Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showyhatch');
            VT := VT + STEP;
          end;

        toly := toly + taly;
      end; {while TOLY}

  end; {PostMarks: Marks and Numbers}

{_________________________________________________}

  procedure PostFrame (var F: text;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX);

    var
      sign, Ycap: integer;
      firstHATCH, xZERO, yZERO, vSTEP, hSTEP: real;
      rxPos, rxNeg, ryPos, ryNeg, taly, increment: real;
      nearBound, farBound, boundRange, startMark: real;

  begin{PostFrame}
    with POSTBOX, dataBOX do
      begin

{(X1,  Y1) is the bottom left of the IBOX}
{(X2,  Y2) is the top lright of the IBOX}

        xZERO := X1; {Default for Graphics Origin. integer to real conversion}
        yZERO := Y1; {Default for Graphics Origin. integer to real conversion}

{BOXWDT := X2 - X1;}
 {Width of Bounding Box: real valued entity}
{BOXHGT := Y2 - Y1; }
 {Height of Bounding Box: : real valued entity}

        if doMarks then
          begin
            Writeln(F, '/Symbol findfont');
            Writeln(F, '10 scalefont  setfont');
          end;
        if doFrame then
          begin
            Writeln(F, X1 : 3, '  ', Y1 : 3, '  moveto');
            Writeln(F, X1 : 3, '  ', Y2 : 3, '  lineto');
            Writeln(F, X2 : 3, '  ', Y2 : 3, '  lineto');
            Writeln(F, X2 : 3, '  ', Y1 : 3, '  lineto');
            Writeln(F, 'closepath');
            Writeln(F, 'stroke');
            Writeln(F, 'newpath');
          end;

        if yScale = true then
          begin {Do the yScale}

 {...............................................................................................}

            if yMin * yMax > 0 then
              begin {yMin, yMax > or yMin, yMax < 0}

                if yMax < 0 then
                  begin
                    sign := -1;
                    nearBound := yMax;
                    farBound := yMin;
                  end
                else
                  begin
                    sign := 1;
                    nearBound := yMin;
                    farBound := yMax
                  end;

                if ((farBound - nearBound) / farBound) < 0.3 then
                  begin {small boundRange}

                    if hBorders then
                      begin
                        nearBound := nearBound - sign * (0.2 * yRange);
                        farBound := farBound + sign * (0.2 * yRange);
                      end;

                    boundRange := Abs(farBound - nearBound);

                    TallyPoints(boundRange, taly);
                    startMark := Trunc(nearBound / taly) * taly;
                    if (Abs((startMark - nearBound) / nearBound) > 0.0001) then
                      startMark := startMark + sign * taly;

                    if sign = 1 then
                      Ycap := Y1
                    else
                      Ycap := Y2;

                    FIRSTHATCH := Ycap + (startMark - nearBound) * BOXHGT / boundRange;
                  end {small boundRange}

                else{large boundRange}
                  begin {yMin = 0.0}

                    if hBorders then {nearBound := 0.0;}
                      farBound := 1.2 * yMax;

                    boundRange := Abs(farBound);
                    TallyPoints(boundRange, taly);
                    startMark := 0.0;
                    FIRSTHATCH := Ycap;
                  end; {large boundRange}

                vSTEP := taly * (BOXHGT / boundRange);
                hSTEP := 0.0;
                if doMarks then
                  PostMarks(F, X1, firstHATCH, sign * taly, startMark, farBound, sign * vSTEP, 'yScale')

              end;{xMin, xMax > or xMin, xMax < 0}
{...............................................................................................}

            if (yMin < 0.0) and (yMax > 0.0) then
              begin {yMin < 0.0}
                ryPos := yMax;
                ryNeg := yMin;

                if hBorders then
                  begin {hBorders}
                    increment := 0.1 * (yMax - yMin);
                    if yMax > Abs(yMin) then
                      begin
                        ryPos := 1.2 * yMax;
                        ryNeg := yMin - increment;
                      end
                    else
                      begin
                        ryNeg := 1.2 * yMin;
                        ryPos := yMax + increment
                      end;
                  end; {hBorders}

                yZERO := Y1 - BOXHGT * ryNeg / (ryPos - ryNeg);
{this draws the horizontal axis through y = 0}

                if (xAxis = true) then
                  begin
                    Writeln(F, X1 : 3, '  ', yZERO : 5 : 2, '  moveto');
                    Writeln(F, X2 : 3, '  ', yZERO : 5 : 2, '  lineto')
                  end;

                if ryPos > Abs(ryNeg) then
                  begin
                    TallyPoints(ryPos, taly);
                    vSTEP := (Y2 - yZERO) * (taly / ryPos);
                  end
                else {if ryPos < Abs(ryNeg)}
                  begin
                    TallyPoints(-ryNeg, taly);
                    vSTEP := (Y1 - yZERO) * taly / ryNeg;
                  end;

                hSTEP := 0.0;
                if doMarks then
                  begin {doMarks}
                    PostMarks(F, X1, yZERO, taly, 0.0, ryPos, vSTEP, 'yScale');
                    PostMarks(F, X1, yZERO, -taly, 0.0, ryNeg, -vSTEP, 'yScale');
                  end; {doMarks}

              end; {yMin < 0.0 and yMax > 0.0}
  {...............................................................................................}

          end; {Do the yScale}
{------------------------------------------------------}

        if xScale then
          begin {Do the xScale}

  {...............................................................................................}

            if (xMin <= 0.0) and (xMax >= 0) then
              begin {xMin <= 0.0}
                rxPos := xMax;
                rxNeg := xMin;

                if (xMin < 0.0) then
                  xZERO := X1 - BOXWDT * rxNeg / (rxPos - rxNeg)
                else
                  xZERO := X1;

                if (xMin < 0.0) and (yAxis = true) then
                  begin  {This provides the vertical axis through x =0}
                    Writeln(F, xZERO : 5 : 2, '  ', Y1 : 3, '  moveto');
                    Writeln(F, xZERO : 5 : 2, '  ', Y2 : 3, '  lineto');
                  end;

                if rxPos > Abs(rxNeg) then
                  begin
                    TallyPoints(rxPos, taly);
                    hSTEP := (X2 - xZERO) * (taly / rxPos);
                  end
                else
                  begin
                    TallyPoints(-rxNeg, taly);
                    hSTEP := (X1 - xZERO) * (taly / rxNeg);
                  end;

                vSTEP := 0;
                if doMarks then
                  begin
                    PostMarks(F, xZERO, Y1, taly, 0, rxPos, hSTEP, 'xScale');
                    PostMarks(F, xZERO, Y1, -taly, 0, rxNeg, -hSTEP, 'xScale');
                  end;

              end; {xMin <= 0.0 and xMax >= 0}

  {...............................................................................................}

            if xMin * xMax > 0.0 then
              begin {xMin, xMax < 0 or xMin, xMax >0}
                sign := 1;
                if xMax < 0 then
                  sign := -1;

                TallyPoints(xRange, taly);
                if xMin > 0 then
                  begin
                    farBound := xMax;
                    nearBound := xMin;
                  end;
                if xMax < 0 then
                  begin
                    farBound := xMin;
                    nearBound := xMax;
                  end;

                startMark := Trunc(nearBound / taly) * taly;
                if (Abs((startMark - nearBound) / nearBound) > 0.001) then
                  startMark := startMark + sign * taly;
                hSTEP := (BOXWDT) * (taly / xRange);
                vSTEP := 0.0;
                if sign = 1 then
                  firstHATCH := X1 + ((startMark - xMin) * BOXWDT) / xRange
                else
                  firstHATCH := X2 - ((xMax - startMark) * BOXWDT) / xRange;
                PostMarks(F, firstHATCH, Y1, sign * taly, startMark, farBound, sign * hSTEP, 'xScale');
              end;

  {...............................................................................................}

            Writeln(F, 'stroke');
            Writeln(F, 'newpath');

          end; {Do the xScale}

{------------------------------------------------------}

      end; {with POSTBOX, dataBOX}

  end; {PostFrame: PostScript Framework}

{________________________________________________}

  procedure PostPlot (var F: text;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting an arbitrary number of data points over}
{the range defined by the width of the bounding box via the}
{index  i := 0 to Tcap - 1: Tcap := dataBOX.NOPUNCT - 1}

    var
      i, Tcap: integer;
      XZERO, YZERO, BOXWDT, BOXHGT: real;
      HR, VT, xRange, yRange: real;

  begin
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XZERO := POSTBOX.X0;
    YZERO := POSTBOX.Y0;
    yRange := dataBOX.yUB - dataBOX.yLB;
    xRange := dataBOX.NOPUNCT - 1; {real value}
    Tcap := dataBOX.NOPUNCT - 1; {Integer Value}

    HR := BOXWDT;
    VT := YZERO + (dataVector[Tcap - 1] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');

    for i := 0 to Tcap - 1 do
      begin {i}
        HR := XZERO + (i * BOXWDT / xRange);
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
        if (i mod 8 = 7) or (i = Tcap - 1) then
          Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2)
        else
          Write(F, HR : 3 : 2, ' ', VT : 3 : 2, ' ');
      end; {i}
    Writeln(F, Tcap : 3, ' {lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostPlot}

{_________________________________________________}

  procedure PostPlotMod (var F: text;
                  var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting a series of function evaluations that are equal}
{in number to the width to the bounding box measured in pixelpoints}
{via the index  i := 0 to BOXWDT - 1}

    var
      i, BOXWDT, BOXHGT, YZERO: integer;
      HR, VT, yRange: real;

  begin
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    YZERO := POSTBOX.Y0;
    yRange := dataBOX.yUB - dataBOX.yLB;

    Writeln(F, 'newpath');
    Writeln(F, '/abscissa ', BOXWDT : 3, ' def');

    HR := BOXWDT;
    VT := YZERO + (dataVector[BOXWDT - 1] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');
{we are plotting in the reverse direction of x}

    for i := 0 to BOXWDT - 1 do
      begin {i}
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
{Here we  impose a limiting value on VT to keep the plot within the  bounding box}
        if VT > POSTBOX.Y2 then
          VT := POSTBOX.Y2;

        if (i mod 8 = 7) then
          Writeln(F, VT : 3 : 2) {this is to break the lines}
        else
          Write(F, VT : 3 : 2, ' ');
      end; {i}
    Write(F, BOXWDT : 3); {repeat these many times}
    Writeln(F, ' {/abscissa abscissa 1 sub def abscissa exch lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostPlotMod}

{_________________________________________________}

  procedure DrawPostSeg (var F: text;
                  var x1, y1, x2, y2: real;
                  var dataBOX: rBox;
                  var POSTBOX: IBOX);

    var
      VT1, VT2, HR1, HR2, temp: real;

  begin

{BOXWDT should be an IBOX parameter}

    with POSTBOX, dataBOX do
      begin
        temp := BOXWDT * (y1 - yMin) / yRange;
        VT1 := Y1 + temp;
        Temp := BOXHGT * (x1 - xMin) / xRange;
        HR1 := X1 + temp;

        temp := BOXWDT * (y2 - yMin) / yRange;
        VT2 := Y1 + temp;
        Temp := BOXHGT * (x2 - xMin) / xRange;
        HR2 := X1 + temp;
      end;

    Writeln(F, HR1 : 3 : 2, '  ', VT1 : 3 : 2, '  moveto');
    Writeln(F, HR2 : 3 : 2, '  ', VT2 : 3 : 2, '  lineto');

  end; {DrawPostSeg}

{_________________________________________________}

end. {POST: Unit 8 of CONTEXT.PAS}

