unit SCREEN; {Unit 9 of CONTEXT.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, Graph, Dos, Crt;

  procedure MarksAndNumbers (XSTART, YSTART: integer;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

  procedure ScreenFrame (var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure ScreenPlot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

  procedure ScreenPlotMod (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

implementation

{_________________________________________________}

  procedure MarksAndNumbers (XSTART, YSTART: integer;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

    var
      toly, HRL, VRL: real;
      HR1, HR2, VT1, VT2, tolyWidth: integer;
      HINC, VINC, PENLEFT, PENDOWN: integer;
      tolyString: string;

  begin {MarksAndNumbers}

    if Axis = 'yScale' then
      begin {Pixel Parameters}
        HINC := 5;
        VINC := 0;
        PENDOWN := 4;
      end;
    if Axis = 'xScale' then
      begin
        HINC := 0;
        VINC := -5;
        PENDOWN := 20;
      end; {Pixel Parameters}

    toly := firstMark;
    HRL := XSTART;
    HR1 := XSTART;
    HR2 := HR1 + HINC;
    VRL := YSTART;
    VT1 := YSTART;
    VT2 := VT1 + VINC;

    while Abs(toly) <= Abs(farBound)+ 0.5 * Abs(taly) do

      begin {while toly}
        tolyString := TalyToString(toly);
        tolyWidth := TextWidth(tolyString);
        if axis = 'xScale' then
          PENLEFT := Round(tolyWidth / 2);
        if axis = 'yScale' then
          PENLEFT := tolyWidth + 15;

 {Numbers}
        MoveTo(HR1 - PENLEFT, VT1 + PENDOWN);
        OutText(tolyString);

  {Hatchmarks}
        MoveTo(HR1, VT1);
        LineTo(HR2, VT2);

  {Increments}
        if axis = 'xScale' then
          HRL := HRL + STEP;
        HR1 := Round(HRL);
        HR2 := HR1 + HINC;
        if axis = 'yScale' then
          VRL := VRL - STEP;
        VT1 := Round(VRL);
        VT2 := VT1 + VINC;

        toly := toly + taly;
      end; {while toly}

  end; {MarksAndNumbers}

{_________________________________________________}

  procedure ScreenFrame (var dataBOX: rBOX;
                  var SCREENBOX: IBOX);


{Uses, MarksAndNumbers, TallyPoints, TalyToString}

    var
      FIRSTHATCH: integer;
      XORIGIN, YORIGIN, YCap, sign: integer;
      rxPos, rxNeg, ryPos, ryNeg, vSTEP, hSTEP, taly: real;
      nearBound, farBound, boundRange, increment, startMark: real;

  begin

    with SCREENBOX, dataBOX do
      begin

{(X1,  Y1) is the top left of the IBOX}
{(X2,  Y2) is the botton lright of the IBOX}

        XORIGIN := X1;{ Default for  XORIGIN}
        YORIGIN := Y2;{ Default for  YORIGIN}

        if doFrame then
          Rectangle(X1, Y1, X2 + 1, Y2 + 1);

        if yScale = true then
          begin {Do the yScale}

{...............................................................................................}

            if yMin * yMax > 0 then
              begin {yMin, yMax > or yMin, yMax < 0}

                if yMax < 0 then
                  begin
                    sign := -1;
                    nearBound := yMax;
                    farBound := yMin;
                  end
                else
                  begin
                    sign := 1;
                    nearBound := yMin;
                    farBound := yMax
                  end;

                if ((farBound - nearBound) / farBound) < 0.3 then
                  begin {small boundRange}

                    if hBorders then
                      begin
                        nearBound := nearBound - sign * (0.2 * yRange);
                        farBound := farBound + sign * (0.2 * yRange);
                      end;

                    boundRange := Abs(farBound - nearBound);

                    TallyPoints(boundRange, taly);
                    startMark := Trunc(nearBound / taly) * taly;
                    if (Abs((startMark - nearBound) / nearBound) > 0.0001) then
                      startMark := startMark + sign * taly;

                    if sign = 1 then
                      Ycap := Y2
                    else
                      Ycap := Y1;

                    FIRSTHATCH := Ycap + Round((nearBound - startMark) * BOXHGT / boundRange);
                  end {small boundRange}

                else{large boundRange}
                  begin {yMin = 0.0}

                    if hBorders then {nearBound := 0.0;}
                      farBound := 1.2 * yMax;

                    boundRange := Abs(farBound);
                    TallyPoints(boundRange, taly);
                    startMark := 0.0;
                    FIRSTHATCH := Ycap;
                  end; {large boundRange}

                VSTEP := taly * (BOXHGT / boundRange);
                HSTEP := 0.0;
                if doMarks then
                  MarksAndNumbers(X1, FIRSTHATCH, sign * taly, startMark, farBound, sign * VSTEP, 'yScale')

              end;{xMin, xMax > or xMin, xMax < 0}
{...............................................................................................}

            if (yMin < 0.0) and (yMax > 0.0) then
              begin {yMin < 0.0}
                ryPos := yMax;
                ryNeg := yMin;

                if hBorders then
                  begin {hBorders}
                    increment := 0.1 * (yMax - yMin);
                    if yMax > Abs(yMin) then
                      begin
                        ryPos := 1.2 * yMax;
                        ryNeg := yMin - increment;
                      end
                    else
                      begin
                        ryNeg := 1.2 * yMin;
                        ryPos := yMax + increment
                      end;
                  end; {hBorders}

                YORIGIN := Y2 + Round(BOXHGT * ryNeg / (ryPos - ryNeg));
{this draws the horizontal axis through y = 0}

                if (xAxis = true) then
                  begin
                    MoveTo(X1, YORIGIN);
                    LineTo(X2, YORIGIN)
                  end;

                if ryPos > Abs(ryNeg) then
                  begin
                    TallyPoints(ryPos, taly);
                    VSTEP := (YORIGIN - Y1) * (taly / ryPos);
                  end
                else {if ryPos < Abs(ryNeg)}
                  begin
                    TallyPoints(-ryNeg, taly);
                    VSTEP := (YORIGIN - Y2) * (taly / ryNeg);
                  end;

                HSTEP := 0.0;
                if doMarks then
                  begin {doMarks}
                    MarksAndNumbers(X1, YORIGIN, taly, 0.0, ryPos, VSTEP, 'yScale');
                    MarksAndNumbers(X1, YORIGIN, -taly, 0.0, ryNeg, -VSTEP, 'yScale');
                  end; {doMarks}

              end; {yMin < 0.0 and yMax > 0.0}
{...............................................................................................}

          end; {Do the yScale}

{-----------------------------------------------}

        if xScale then
          begin {Do the xScale}

{...............................................................................................}

            if (xMin <= 0.0) and (xMax >= 0) then
              begin {xMin <= 0.0}
                rxPos := xMax;
                rxNeg := xMin;

                if (xMin < 0.0) then
                  XORIGIN := X1 - Round(BOXWDT * rxNeg / (rxPos - rxNeg))
                else
                  XORIGIN := X1;

                if (xMin < 0.0) and (yAxis = true) then
                  begin  {This provides the vertical axis through x =0}
                    MoveTo(XORIGIN, Y1);
                    LineTo(XORIGIN, Y2)
                  end;

                if rxPos > Abs(rxNeg) then
                  begin
                    TallyPoints(rxPos, taly);
                    HSTEP := (X2 - XORIGIN) * (taly / rxPos);
                  end
                else
                  begin
                    TallyPoints(-rxNeg, taly);
                    HSTEP := (X1 - XORIGIN) * (taly / rxNeg);
                  end;

                VSTEP := 0;
                if xMax > 0.0 then
                  MarksAndNumbers(XORIGIN, Y2, taly, 0, rxPos, HSTEP, 'xScale');
                if xMin < 0.0 then
                  MarksAndNumbers(XORIGIN, Y2, -taly, 0, rxNeg, -HSTEP, 'xScale');
              end; {xMin <= 0.0}

{...............................................................................................}

            if xMin * xMax > 0.0 then
              begin {xMin, xMax < 0 or xMin, xMax >0}
                sign := 1;
                if xMax < 0 then
                  sign := -1;

                TallyPoints(xRange, taly);
                if xMin > 0 then
                  begin
                    farBound := xMax;
                    nearBound := xMin;
                  end;
                if xMax < 0 then
                  begin
                    farBound := xMin;
                    nearBound := xMax;
                  end;

                startMark := Trunc(nearBound / taly) * taly;
                if (Abs((startMark - nearBound) / nearBound) > 0.0001) then
                  startMark := startMark + sign * taly;
                HSTEP := (BOXWDT) * (taly / xRange);
                VSTEP := 0.0;
                if sign = 1 then
                  FIRSTHATCH := X1 + Round((startMark - xMin) * BOXWDT / xRange)
                else
                  FIRSTHATCH := X2 - Round((xMax - startMark) * BOXWDT / xRange);
                MarksAndNumbers(FIRSTHATCH, Y2, sign * taly, startMark, farBound, sign * HSTEP, 'xScale');
              end;
{...............................................................................................}

          end; {Do the xScale}

{-----------------------------------------------}

      end; {with SCREENBOX and dataBOX}

  end; {ScreenFrame}

{________________________________________________}

  procedure ScreenPlot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting an arbitrary number of data points }
{over the range defined by the width of the bounding box}

    var
      i, HR, VT, BOXHGT: integer;
      xRange, yRange, BOXWDT: real;
      store: double;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := dataBOX.yUB - dataBOX.yLB;
    xRange := dataBOX.xRB - dataBOX.xLB;

    for i := 0 to dataBOX.NOPUNCT - 1 do
      begin {i}
        store := i * BOXWDT;
        store := store / (dataBOX.NOPUNCT - 1);
        HR := SCREENBOX.X1 + Round(store);

        store := (dataVector[i] - dataBOX.yLB) / yRange;
        VT := Round(SCREENBOX.Y2 - store * BOXHGT);
        if VT < SCREENBOX.Y1 then
          VT := SCREENBOX.Y1;
        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}

  end; {ScreenPlot}

{_________________________________________________}

  procedure ScreenPlotMod (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting a series of function evaluations thar are equal}
{in number to the width to the bounding box measured in pixelpoints.}

    var
      i, HR, VT, XORIGIN, YORIGIN, BOXWDT, BOXHGT: integer;
      store, yRange: real;

  begin
    yRange := dataBOX.yUB - dataBOX.yLB;
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    YORIGIN := Round(SCREENBOX.Y0);
    XORIGIN := Round(SCREENBOX.X0);


    for i := 0 to BOXWDT do
      begin {i}
        HR := XORIGIN + i;
        store := dataVector[i] * (BOXHGT / yRange);
        VT := YORIGIN - Round(store);
        if VT < SCREENBOX.Y1 then
          VT := SCREENBOX.Y1;
        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}
  end; {ScreenPlotMod}

{_________________________________________________}

end. {SCREEN: Unit 9 of CONTEXT.PAS}

